{{
  Buffered String Format Object

  This object mirrors all the functions of StrFmt, but
  does not require the caller to supply a buffer.  The
  supplied buffers are eventually recycled, and the
  caller needs to be aware of how much space is available
  when pushing the life span of returned results.

  To facilitate saving Hub RAM this object requires the
  buffer to be externally supplied.  It is highly
  recommended to recycle a PASM image for this purpose
  after another object has used it to start a cog.

  Unlike StrFmt this object is not multi-cog thread safe,
  but it would be if the bufalloc() function were protected
  by a lock or semaphore.
}}

obj

  sf : "strfmt"

con

  TbufLen = 64

dat

bufbase word 0
bufend  word 0
bufptr  word 0

pub setbuf(b_base, b_end)

  bufbase := b_base
  bufend := b_end
  bufptr := b_base

pub bufstr(ptr) | l
{
  Copy a string to a return buffer
}
  l := strsize(ptr) + 1
  return bufbuf(ptr, l)

PUB bufbuf(ptr, l)
{
  Copy a possibly non-null-terminated buffer
  to a return buffer.  This does NOT null
  terminate it if it isn't already.
}
  result := bufalloc(l)
  bytemove(result, ptr, l)

PUB bufalloc(l)
{
  Reserve a return buffer of the given length
}
  if bufptr + l => bufend
    bufptr := bufbase
  byte[bufptr] := 0       'null terminate!
  result := bufptr
  bufptr += l

pub strcat(sptr1, sptr2) : ptr
{
  Return two null-terminated strings concatenated; note
  this is not quite the same relationship to sf.cat()
  as with the other mirrored functions.
}
  ptr := bufalloc(strsize(sptr1)+strsize(sptr2)+1)
  sf.str(ptr, sptr1)
  sf.cat(ptr, sptr2)

pub chcat(sptr, a) : ptr
{
  Return a string with the character appended to it.
  Note that this really churns the return buffer.
}
  ptr := bufalloc(strsize(sptr)+2)
  sf.str(ptr, sptr)
  sf.chcat(ptr, a)
  
{{
  Mirrored strfmt functions
}}

pub ch(a) | buf

  sf.ch(@buf, a)
  return bufstr(@buf)

pub chstr(a, len) | buf[tbuflen]

  sf.chstr(@buf, a, len)
  return bufstr(@buf)

pub str(sptr)

  return bufstr(sptr)

pub lstr(sptr, len) | buf[tbuflen] 

  sf.lstr(@buf, sptr, len)
  return bufstr(@buf)

pub lstrpad(sptr, len, padchr) | buf[tbuflen] 
  
  sf.lstrpad(@buf, sptr, len, padchr)
  return bufstr(@buf)

pub rstr(sptr, len) | buf[tbuflen] 

  sf.rstr(@buf, sptr, len)
  return bufstr(@buf)

pub rstrpad(sptr, len, padchr) | buf[tbuflen] 
  
  sf.rstrpad(@buf, sptr, len, padchr)
  return bufstr(@buf)

pub cstr(sptr, len) | buf[tbuflen] 

  sf.cstr(@buf, sptr, len)
  return bufstr(@buf)

pub cstrpad(sptr, len, padchr) | buf[tbuflen] 
  
  sf.cstrpad(@buf, sptr, len, padchr)
  return bufstr(@buf)

pub dec(value) | buf[tbuflen]

  sf.dec(@buf, value)
  return bufstr(@buf)

pub rdec(value, len) | buf[tbuflen]

  sf.rdec(@buf, value, len)
  return bufstr(@buf)

pub hex(value, digits) | buf[tbuflen]

  sf.hex(@buf, value, digits)
  return bufstr(@buf)

pub bin(value, digits) | buf[tbuflen]

  sf.bin(@buf, value, digits)
  return bufstr(@buf)

pub fdec(value, len, dp) | buf[tbuflen]

  sf.fdec(@buf, value, len, dp)
  return bufstr(@buf)

pub val(ptr)

  return sf.val(ptr)  